package org.tc.autonomous;

import com.fs.starfarer.api.GameState;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CargoAPI.CrewXPLevel;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.combat.AssignmentTargetAPI;
import com.fs.starfarer.api.combat.CombatAssignmentType;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.CombatFleetManagerAPI;
import com.fs.starfarer.api.combat.CombatTaskManagerAPI;
import com.fs.starfarer.api.combat.EveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipHullSpecAPI.ShipTypeHints;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.combat.ShipwideAIFlags.AIFlags;
import com.fs.starfarer.api.combat.ShipVariantAPI;
import com.fs.starfarer.api.combat.ViewportAPI;
import com.fs.starfarer.api.fleet.FleetGoal;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.input.InputEventAPI;
import com.fs.starfarer.api.mission.FleetSide;
import com.fs.starfarer.api.util.IntervalUtil;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;
import java.awt.Color;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.log4j.Logger;
import org.json.JSONObject;
import org.lwjgl.util.vector.Vector2f;


public class AutonomousCommandsPlugin implements EveryFrameCombatPlugin {

	private static Logger LOG = Global.getLogger(AutonomousCommandsPlugin.class);
	
	private static Color TEXT_COLOR = Global.getSettings().getColor("standardTextColor");
	private static Color SHIP_COLOR = Global.getSettings().getColor("textFriendColor");
	private static Color MESSAGE_COLOR = Color.CYAN;

	public static final String CONFIG_FILE = "data/config/autonomous-ships.json";
	public static final String RETREAT_COMMAND_PREFIX = "autonomous_retreat_";

	public static Map<String, Double> retreatHullMods = new HashMap<>();
	static {
		try {
			LOG.info("Loading autonomous commands configuration from " + CONFIG_FILE + " ...");
			JSONObject settings = Global.getSettings().loadJSON(CONFIG_FILE);
			for (int i = 1; i <= 3; i++) {
				String key = RETREAT_COMMAND_PREFIX + i;
				double value = settings.getDouble(key);
				retreatHullMods.put(key, value);
			}
			LOG.info("Retreat command hullmods: " + retreatHullMods);
		}
		catch (Exception e) {
			throw new IllegalStateException(e);
		}
	}

	private CombatEngineAPI engine;

	@Override
	public void advance(float amount, List<InputEventAPI> events) {
		if (engine == null) return;
		if (engine.isPaused()) return;
		if (engine.isSimulation()) return;
		if (Global.getCurrentState() == GameState.TITLE) return;
		
		// TODO(hqz) only act every second or so.

		CombatFleetManagerAPI fleetManager = engine.getFleetManager(FleetSide.PLAYER);
		CombatTaskManagerAPI taskManager = fleetManager.getTaskManager(false);
		for (FleetMemberAPI member : fleetManager.getDeployedCopy())
		{
			if (member.isFighterWing()) continue;
			if (member.isFlagship()) continue;
			if (member.isAlly()) continue;

			ShipAPI ship = fleetManager.getShipFor(member);
			if (ship.getShipAI() == null) continue;
			if (!ship.isAlive()) continue;

			float hull = ship.getHullLevel();
			if (hull == 1.0) return;

			ShipVariantAPI variant = ship.getVariant();

			for (Map.Entry<String, Double> hullMod : retreatHullMods.entrySet()) {
				if (variant.hasHullMod(hullMod.getKey()) && hull < hullMod.getValue()) {
					CombatFleetManagerAPI.AssignmentInfo assignment = taskManager.getAssignmentFor(ship);
					if (assignment == null || assignment.getType() != CombatAssignmentType.RETREAT) {
						LOG.info(ship.getName() + " retreating (hull = " + hull + ")");
						String message = String.format("hull integrity is %d%%, retreating ...", (int)(hull * 100f));
						engine.getCombatUI().addMessage(1, member, SHIP_COLOR, ship.getName(), TEXT_COLOR, ": ", MESSAGE_COLOR, message);
						taskManager.orderRetreat(fleetManager.getDeployedFleetMember(ship), false);
						// TODO(hqz) this is for debug
						CombatFleetManagerAPI.AssignmentInfo retreat = taskManager.getAssignmentFor(ship);
						LOG.info(retreat.getTarget());
					}
					break;
				}
			}
		}
	}

	@Override
	public void renderInWorldCoords(ViewportAPI vapi) {
	}

	@Override
	public void renderInUICoords(ViewportAPI vapi) {
	}

	@Override
	public void init(CombatEngineAPI engine) {
		this.engine = engine;
		LOG.info("Autonomous commands plugin initialized");
	}
}
